# -*- coding: utf-8 -*-
"""
/***************************************************************************
 BackupLayer
                                 A QGIS plugin
 Backup the selected layer by saving the corresponding data file(s) in a zip file with datetime stamp appended to name
                              -------------------
        begin                : 2015-11-17
        git sha              : $Format:%H$
        copyright            : (C) 2015 by Josef Källgården
        email                : groundwatergis@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
from PyQt4.QtCore import QSettings, QTranslator, qVersion, QCoreApplication
from PyQt4.QtGui import QAction, QIcon, QDialog, QMessageBox, QPushButton
# Initialize Qt resources from file resources.py
import resources
# Import some general python modules
import os.path
import sys
import datetime
import zipfile
try:
    import zlib
    compression = zipfile.ZIP_DEFLATED
except:
    compression = zipfile.ZIP_STORED
# Import the code for the dialog
import os


class BackupLayer:
    """QGIS Plugin Implementation."""

    def __init__(self, iface):
        """Constructor.

        :param iface: An interface instance that will be passed to this class
            which provides the hook by which you can manipulate the QGIS
            application at run time.
        :type iface: QgsInterface
        """
        # Save reference to the QGIS interface
        self.iface = iface
        # initialize plugin directory
        self.plugin_dir = os.path.dirname(__file__)

        # initialize locale
        locale = QSettings().value('locale/userLocale')[0:2]
        locale_path = os.path.join(
            self.plugin_dir,
            'i18n',
            'BackupLayer_{}.qm'.format(locale))

        if os.path.exists(locale_path):
            self.translator = QTranslator()
            self.translator.load(locale_path)

            if qVersion() > '4.3.3':
                QCoreApplication.installTranslator(self.translator)
                        
        # Declare instance attributes
        self.actions = []
        self.menu = self.tr(u'&Backup layer')


    # noinspection PyMethodMayBeStatic
    def tr(self, message):
        """Get the translation for a string using Qt translation API.

        We implement this ourselves since we do not inherit QObject.

        :param message: String for translation.
        :type message: str, QString

        :returns: Translated version of message.
        :rtype: QString
        """
        # noinspection PyTypeChecker,PyArgumentList,PyCallByClass
        return QCoreApplication.translate('BackupLayer', message)


    def add_action(
        self,
        icon_path,
        text,
        callback,
        enabled_flag=True,
        add_to_menu=True,
        add_to_toolbar=True,
        status_tip=None,
        whats_this=None,
        parent=None):
        """Add a toolbar icon to the toolbar.

        :param icon_path: Path to the icon for this action. Can be a resource
            path (e.g. ':/plugins/foo/bar.png') or a normal file system path.
        :type icon_path: str

        :param text: Text that should be shown in menu items for this action.
        :type text: str

        :param callback: Function to be called when the action is triggered.
        :type callback: function

        :param enabled_flag: A flag indicating if the action should be enabled
            by default. Defaults to True.
        :type enabled_flag: bool

        :param add_to_menu: Flag indicating whether the action should also
            be added to the menu. Defaults to True.
        :type add_to_menu: bool

        :param add_to_toolbar: Flag indicating whether the action should also
            be added to the toolbar. Defaults to True.
        :type add_to_toolbar: bool

        :param status_tip: Optional text to show in a popup when mouse pointer
            hovers over the action.
        :type status_tip: str

        :param parent: Parent widget for the new action. Defaults None.
        :type parent: QWidget

        :param whats_this: Optional text to show in the status bar when the
            mouse pointer hovers over the action.

        :returns: The action that was created. Note that the action is also
            added to self.actions list.
        :rtype: QAction
        """

        icon = QIcon(icon_path)
        action = QAction(icon, text, parent)
        action.triggered.connect(callback)
        action.setEnabled(enabled_flag)

        if status_tip is not None:
            action.setStatusTip(status_tip)

        if whats_this is not None:
            action.setWhatsThis(whats_this)

        if add_to_toolbar:
            # Add toolbar button and menu item
            self.iface.addToolBarIcon(action)

        if add_to_menu:
            self.iface.addPluginToMenu(
                self.menu,
                action)

        self.actions.append(action)

        return action


    def initGui(self):
        """Create the menu entries and toolbar icons inside the QGIS GUI."""

        icon_path = ':/plugins/BackupLayer/icon.png'
        self.add_action(
            icon_path,
            text=self.tr(u'zip selected layer'),
            callback=self.run,
            parent=self.iface.mainWindow())


    def unload(self):
        """Removes the plugin menu item and icon from QGIS GUI."""
        for action in self.actions:
            self.iface.removePluginMenu(
                self.tr(u'&Backup layer'),
                action)
            self.iface.removeToolBarIcon(action)


    def run(self):
        """Run method that performs all the real work"""
        lyr = self.iface.activeLayer()
        if lyr is not None:
            (fpath,fname) = os.path.split(lyr.dataProvider().dataSourceUri())
            fname = os.path.splitext(fname)[0]
            bkupname = os.path.join(fpath, fname + datetime.datetime.now().strftime('%Y%m%dT%H%M') + '.zip')
            sanity = askuser("""This will create a backup file named """ + bkupname + """\n\nContinue?""",'Backup?')
            # See if OK was pressed
            if sanity.result == 1:
                try:
                    zf = zipfile.ZipFile(bkupname, mode='w')
                    for file in os.listdir(fpath):
                        BaseName = os.path.basename(file)
                        FileName, FileExtension = os.path.splitext(BaseName)
                        if FileName==fname:
                            print('compressing ' + os.path.join(fpath,BaseName))
                            zf.write(os.path.join(fpath,BaseName),BaseName, compress_type=compression) #compression will depend on if zlib is found or not
                    zf.close()
                except:
                    self.iface.messageBar().pushMessage("Error","Problems creating archive. Verify this is a file layer and not a database layer.", 2, duration=5)
        else:
            self.iface.messageBar().pushMessage("Error","Select a layer!", 2, duration=5)

class askuser(QDialog):
    def __init__(self, msg = '', dialogtitle='User input needed', parent=None):
        self.result = ''
        reply = QMessageBox.information(parent, dialogtitle, msg, QMessageBox.Yes | QMessageBox.No, QMessageBox.Yes)
        if reply==QMessageBox.Yes:
            self.result = 1 #1 = "yes"
        else:
            self.result = 0  #0="no"
